
;HexDump1.inc, , DednDave updated Dec 12, 2011

;Hexidecimal Dump Routines
;
;These routines use the external functions PrintString and PrintNewLine.
;
;Conversion Functions Provided: Byte2Hex
;                               Word2Hex
;                               Dword2Hex
;
;   Display Functions Provided: PrintHexB
;                               PrintHexW
;                               PrintHexD
;
;      Dump Functions Provided: DumpBytes
;                               DumpWords
;                               DumpDwords
;                               DumpRegs
;
;  For the dump functions, all registers and flags are preserved.
;This facilitates the use of these routines for debugging as well
;as for learning aide purposes, as the CALL's may be placed
;anywhere in code with minimal modification. However, this has no
;meaning for the EAX and ECX registers when using the memory dump
;functions, as they are used to pass parameters. To use one of the
;memory dump functions without modifying any registers or flags,
;simply PUSH and POP the contents of EAX and ECX as shown in the
;following example:
;
;       push    eax
;       push    ecx
;       mov     eax, offset MemoryBytes
;       mov     ecx, NumberOfBytesToDump
;       call    DumpBytes
;       pop     ecx
;       pop     eax

;##################################################################

        .DATA

;register name strings

szEax   db 'EAX=', 0
szEbx   db 'EBX=', 0
szEcx   db 'ECX=', 0
szEdx   db 'EDX=', 0
szEip   db 'EIP=', 0
szEsi   db 'ESI=', 0
szEdi   db 'EDI=', 0
szEbp   db 'EBP=', 0
szEsp   db 'ESP=', 0

;flag state strings

szCf0   db 'NC', 0                      ;carry flag = 0
szCf1   db 'CY', 0                      ;carry flag = 1
szOf0   db 'NV', 0                      ;overflow flag = 0
szOf1   db 'OV', 0                      ;overflow flag = 1
szSf0   db 'PL', 0                      ;sign flag = 0
szSf1   db 'NG', 0                      ;sign flag = 1
szZf0   db 'NZ', 0                      ;zero flag = 0
szZf1   db 'ZR', 0                      ;zero flag = 1
szPf0   db 'PO', 0                      ;parity flag = 0
szPf1   db 'PE', 0                      ;parity flag = 1
szDf0   db 'UP', 0                      ;direction flag = 0
szDf1   db 'DN', 0                      ;direction flag = 1

;misc strings

szHyph  db '-', 0                       ;hyphen
szPerio db '.', 0                       ;period
szUnder db '__', 0                      ;2 underscores
szSpace db 3 dup(32), 0                 ;3 spaces

;##################################################################

        .CODE

;******************************************************************

HexByte PROC

;Convert a Byte to ASCII Hexidecimal Characters
;
;  This routine is designed to be used as part of the byte, word,
;and dword routines. The address passed to the routine is that of
;the byte after the second byte to be placed in the string. The
;address register, EDX, is decremented twice. Also, EAX is
;preserved across the call so that the word or dword contents are
;not lost. Refer to the code in the Byte2Hex, Word2Hex, and
;Dword2Hex routines for further details.
;
;Call With:  AL = byte to convert
;           EDX = address in string after second byte
;
;  Returns: EAX = preserved
;           EDX = EDX-2, address of first byte placed into string
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------

        mov     cl, al
        dec     edx
        shr     ecx, 4                  ;CL = high 4 bits
        mov     ch, al                  ;CH = low 4 bits
        dec     edx
        and     cx, 0F0Fh               ;remove unwanted bits
        or      cx, 3030h               ;convert to ASCII
        cmp     cl, 39h                 ;is it above '9' ?
        jbe     HexBy0                  ;no - test CH character

        add     cl, 7                   ;yes - adjust to 'A'-'F'

HexBy0: cmp     ch, 39h                 ;is it above '9' ?
        jbe     HexBy1                  ;no - store the characters

        add     ch, 7                   ;yes - adjust to 'A'-'F'

HexBy1: mov     [edx], cx               ;store both characters

        ret

HexByte ENDP

;******************************************************************

Byte2Hex PROC

;Convert a Byte to ASCII Hexidecimal String
;
;Call With:  AL = byte value to be converted
;
;  Returns: EAX = EDX = address of ASCII hex string
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------

        mov     edx, offset OutBuf+sizeof OutBuf-1
        call    HexByte
        mov     eax, edx

        ret

Byte2Hex ENDP

;******************************************************************

Word2Hex PROC

;Convert a Word to ASCII Hexidecimal String
;
;Call With:  AX = word value to be converted
;
;  Returns: EAX = EDX = address of ASCII hex string
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------

        mov     edx, offset OutBuf+sizeof OutBuf-1
        call    HexByte
        mov     al, ah
        call    HexByte
        mov     eax, edx

        ret

Word2Hex ENDP

;******************************************************************

Dword2Hex PROC

;Convert a Dword to ASCII Hexidecimal String
;
;Call With: EAX = dword value to be converted
;
;  Returns: EAX = EDX = address of ASCII hex string
;           ECX = 0
;
;Also Uses: all other registers are preserved

;------------------------------------------------------------------

        mov     ecx, 4
        mov     edx, offset OutBuf+sizeof OutBuf-1

D2Hex0: push    ecx
        call    HexByte
        pop     ecx
        ror     eax, 8
        loop    D2Hex0

        mov     eax, edx

        ret

Dword2Hex ENDP

;******************************************************************

PrintHexB PROC

;Display a Byte as an ASCII Hexidecimal String to Console
;
;Call With:  AL = byte value to be displayed
;
;  Returns: EAX = number of characters displayed (always 2)
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

        call    Byte2Hex
        jmp     PrintString

PrintHexB ENDP

;******************************************************************

PrintHexW PROC

;Display a Word as an ASCII Hexidecimal String to Console
;
;Call With:  AX = word value to be displayed
;
;  Returns: EAX = number of characters displayed (always 4)
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

        call    Word2Hex
        jmp     PrintString

PrintHexW ENDP

;******************************************************************

PrintHexD PROC

;Display a Dword as an ASCII Hexidecimal String to Console
;
;Call With: EAX = dword value to be displayed
;
;  Returns: EAX = number of characters displayed (always 8)
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

        call    Dword2Hex
        jmp     PrintString

PrintHexD ENDP

;******************************************************************

DumpBytes PROC

;Dump Contents of Memory in "Debug" Style Format, as Bytes
;
;Call With: EAX = start address
;           ECX = number of bytes to dump
;
;  Returns: The requested bytes are dumped to the console display
;
;     Uses: All registers and flags are preserved

;------------------------------------------------------------------

        pushfd
        or      ecx, ecx
        jz      DumpB7

        pushad

        mov     ebp, eax                ;EBP = address of first byte
        mov     ebx, ecx
        dec     ebx
        add     ebx, eax                ;EBX = address of last byte

        and     eax, 0FFFFFFF0h
        mov     edi, ebx
        mov     esi, eax                ;ESI = current address
        sub     edi, eax
        shr     edi, 4
        inc     edi                     ;EDI = number of lines to display

DumpB0: push    esi

;display address in hex

        call    PrintNewLine            ;start new line
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        mov     eax, esi                ;display the line address in hex
        call    PrintHexD
        mov     eax, offset szSpace+1   ;display 2 spaces
        call    PrintString

;display first group of 8 data bytes in hex

        mov     ecx, 8

DumpB1: push    ecx
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        mov     eax, offset szUnder     ;EAX = address of 2 underscores
        cmp     esi, ebp                ;if below first byte
        jb      DumpB2                  ;display underscores

        cmp     esi, ebx                ;if above last byte
        ja      DumpB2                  ;display underscores

        mov     al, [esi]               ;otherwise, get byte data
        call    Byte2Hex                ;display data byte in hex

DumpB2: call    PrintString
        pop     ecx
        inc     esi
        loop    DumpB1

;display a hyphen

        mov     eax, offset szHyph      ;display hyphen
        call    PrintString

;display second group of 8 data bytes in hex

        mov     ecx, 8

DumpB3: push    ecx
        mov     eax, offset szUnder     ;EAX = address of 2 underscores
        cmp     esi, ebp                ;if below first byte
        jb      DumpB4                  ;display underscores

        cmp     esi, ebx                ;if above last byte
        ja      DumpB4                  ;display underscores

        mov     al, [esi]               ;otherwise, get byte data
        call    Byte2Hex                ;display data byte in hex

DumpB4: call    PrintString
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        pop     ecx
        inc     esi
        loop    DumpB3

;display 2 spaces

        mov     eax, offset szSpace+1
        call    PrintString

;display 16 data bytes as ASCII text

        pop     esi
        mov     ecx, 16

DumpB5: push    ecx
        mov     eax, offset szSpace+2   ;EAX = address of 1 space
        cmp     esi, ebp                ;if below first byte
        jb      DumpB6                  ;display a space

        cmp     esi, ebx                ;if above last byte
        ja      DumpB6                  ;display a space

        mov     dl, [esi]
        mov     eax, offset szPerio     ;EAX = address of a period
        cmp     dl, 32                  ;if it is below ASCII space
        jb      DumpB6                  ;display a period

        mov     eax, offset OutBuf+sizeof OutBuf-2
        mov     [eax], dl               ;display the data byte as ASCII

DumpB6: call    PrintString
        pop     ecx
        inc     esi
        loop    DumpB5

;end of loop - test for next line

        dec     edi
        jnz     DumpB0

        call    PrintNewLine

        popad

DumpB7: popfd

        ret

DumpBytes ENDP

;******************************************************************

DumpWords PROC

;Dump Contents of Memory in "Debug" Style Format, as Words
;
;Call With: EAX = start address
;           ECX = number of words to dump
;
;  Returns: The requested words are dumped to the console display
;
;     Uses: All registers and flags are preserved

;------------------------------------------------------------------

        pushfd
        or      ecx, ecx
        jz      DumpW3

        pushad

        mov     edi, ecx
        mov     esi, eax                ;ESI = current address
        dec     edi
        shl     ecx, 1                  ;convert word count to byte count
        mov     ebx, ecx
        shr     edi, 2
        add     ebx, eax                ;EBX = address after last word
        inc     edi                     ;EDI = number of lines to display

;display line address in hex

DumpW0: call    PrintNewLine            ;start new line
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        mov     eax, esi                ;display the line address in hex
        call    PrintHexD
        mov     eax, offset szSpace+1   ;display 2 spaces
        call    PrintString

;display up to 8 words in hex

        mov     ecx, 8

DumpW1: cmp     esi, ebx                ;if above last dword
        jae     DumpW2                  ;display nothing - exit loop

        push    ecx
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        lodsw                           ;get word data
        call    PrintHexW
        pop     ecx
        loop    DumpW1

;end of loop - test for next line

        dec     edi
        jnz     DumpW0

DumpW2: call    PrintNewLine

        popad

DumpW3: popfd

        ret

DumpWords ENDP

;******************************************************************

DumpDwords PROC

;Dump Contents of Memory in "Debug" Style Format, as Dwords
;
;Call With: EAX = start address
;           ECX = number of dwords to dump
;
;  Returns: The requested dwords are dumped to the console display
;
;     Uses: All registers and flags are preserved

;------------------------------------------------------------------

        pushfd
        or      ecx, ecx
        jz      DumpD3

        pushad

        mov     edi, ecx
        mov     esi, eax                ;ESI = current address
        dec     edi
        shl     ecx, 2                  ;convert dword count to byte count
        mov     ebx, ecx
        shr     edi, 2
        add     ebx, eax                ;EBX = address after last dword
        inc     edi                     ;EDI = number of lines to display

;display line address in hex

DumpD0: call    PrintNewLine            ;start new line
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        mov     eax, esi                ;display the line address in hex
        call    PrintHexD
        mov     eax, offset szSpace+1   ;display 2 spaces
        call    PrintString

;display up to 4 dwords in hex

        mov     ecx, 4

DumpD1: cmp     esi, ebx                ;if above last dword
        jae     DumpD2                  ;display nothing - exit loop

        push    ecx
        mov     eax, offset szSpace+2   ;display 1 space
        call    PrintString
        lodsd                           ;get dword data
        call    PrintHexD
        pop     ecx
        loop    DumpD1

;end of loop - test for next line

        dec     edi
        jnz     DumpD0

DumpD2: call    PrintNewLine

        popad

DumpD3: popfd

        ret

DumpDwords ENDP

;******************************************************************

DumpRegs PROC

;Dump Register and Flag Contents in "Debug" Style Format
;
;Call With: Nothing
;
;  Returns: The contents of relevant registers and flags
;           are dumped to the console display
;
;     Uses: All registers and flags are preserved

;------------------------------------------------------------------

        pushad                          ;push 8 registers
        pushfd                          ;push eflags register

        mov     ebp,esp                 ;EBP = base pointer
        cld                             ;direction = up

;[EBP]    = EFlags            from PUSHFD
;[EBP+4]  = EDI               from PUSHAD
;[EBP+8]  = ESI               from PUSHAD
;[EBP+12] = EBP               from PUSHAD
;[EBP+16] = ESP-4             from PUSHAD
;[EBP+20] = EBX               from PUSHAD
;[EBP+24] = EDX               from PUSHAD
;[EBP+28] = ECX               from PUSHAD
;[EBP+32] = EAX               from PUSHAD
;[EBP+36] = RET address (EIP) from CALL DumpRegs

;  Here, we want to re-order the register content values to be convenient
;for our display loop. We do this by PUSH'ing the 9 dword values onto the
;stack in the order that is reverse that to be displayed. As each pass of
;the display loop is executed, we POP one value from the stack to be
;displayed at the appropriate time.
;  When the routine is CALL'ed, the RETurn address is PUSH'ed onto the stack.
;We want to display the value of ESP prior to this PUSH, so it is adjusted by 4.
;  Also, notice that the RETurn address that is PUSH'ed during the CALL is the
;address of the instruction following the CALL instruction. This is the value
;that we display as the contents of the EIP register (Extended Instruction
;Pointer). We know that the CALL instruction is 5 bytes in length. If we wanted
;to display the contents of EIP prior to the CALL, we would subtract 5.

        mov     eax, [ebp+16]           ;original ESP contents, less 4 for RET address
        add     eax, 4                  ;original ESP contents
        push    eax
        push    [ebp+12]                ;original EBP contents
        push    [ebp+4]                 ;original EDI contents
        push    [ebp+8]                 ;original ESI contents
        push    [ebp+36]                ;EIP contents after RETurn
        push    [ebp+24]                ;original EDX contents
        push    [ebp+28]                ;original ECX contents
        push    [ebp+20]                ;original EBX contents
        push    [ebp+32]                ;original EAX contents

;This loop displays the register names and values.

        mov     ebx, 9                  ;number of registers to display
        mov     esi, offset szEax       ;address of register name strings

DumpR0: cmp     ebx, 9                  ;before the first...
        jz      DumpR1

        cmp     ebx, 4                  ;and the sixth register name...
        jnz     DumpR2

DumpR1: call    PrintNewLine            ;...start a new line...
        mov     eax, offset szSpace+2   ;...and display 1 space
        call    PrintString

DumpR2: mov     eax, esi
        call    PrintString             ;display the register name
        pop     eax                     ;recall register content from stack
        call    PrintHexD
        mov     eax, offset szSpace     ;display 3 spaces after each register value
        call    PrintString
        add     esi, 5                  ;each name string is 4 characters plus null
        dec     ebx
        jnz     DumpR0

;  Again, we want to re-order the information given to us, but this time it
;is bits instead of bytes. We want 6 of the bits from the EFlags register to
;be copied into EBX. We use the BT (Bit Test) instruction which copies the
;bit into the carry flag. Then, the result from the carry flag is rotated
;into EBX using the RCL (Rotate through Carry Left) instruction. When the
;loop executes, the bits are shifted from EBX into the carry flag and used
;to select the appropriate flag-state string to be displayed.

        xor     ebx, ebx
        mov     eax, [ebp]              ;original EFlags contents
        bt      eax, 10                 ;EFlags bit 10 = direction flag
        rcl     ebx, 1
        bt      eax, 2                  ;EFlags bit 2 = parity flag
        rcl     ebx, 1
        bt      eax, 6                  ;EFlags bit 6 = zero flag
        rcl     ebx, 1
        bt      eax, 7                  ;EFlags bit 7 = sign flag
        rcl     ebx, 1
        bt      eax, 11                 ;EFlags bit 11 = overflow flag
        rcl     ebx, 1
        bt      eax, 0                  ;EFlags bit 0 = carry flag
        rcl     ebx, 1

        mov     ecx, 6                  ;number of flags to display
        mov     eax, offset szCf0       ;address of flag-state strings

DumpR3: push    ecx
        push    eax
        shr     ebx, 1
        jnc     DumpR4

        add     eax, 3                  ;if flag is set, select alternate string

DumpR4: call    PrintString
        mov     eax, offset szSpace+2   ;display 1 space between each flag
        call    PrintString
        pop     eax
        pop     ecx
        add     eax, 6                  ;each string = 3 bytes, 2 strings per flag
        loop    DumpR3

        call    PrintNewLine

        popfd
        popad

        ret

DumpRegs ENDP

;##################################################################

;end of HexDump1.inc file

